SystemOrganization addCategory: #'Comet-Core'!

!GRPlatform methodsFor: '*comet-core' stamp: 'jf 2/9/2010 01:26'!
cometForkProcessDo: aBlock
	self subclassResponsibility! !

!GRPlatform methodsFor: '*comet-core' stamp: 'jf 2/9/2010 01:26'!
cometWait
	"Pause the process for a period of time before pinging the instances again."

	self subclassResponsibility! !

WARequestFilter subclass: #CTFilter
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTFilter methodsFor: 'processing' stamp: 'lr 3/5/2009 21:37'!
cometForContext: aRequestContext
	^ (handlers
		at: (aRequestContext request fields
			at: CTComet headerField
			ifAbsent: [ ^ nil ])
		ifAbsent: [ ^ nil ])
			connect: aRequestContext! !

!CTFilter methodsFor: 'processing' stamp: 'jf 1/11/2010 12:20'!
handleFiltered: aRequestContext
	(aRequestContext response isKindOf: WAStreamedResponse)
		ifFalse: [ CTStreamingServerRequired signal: 'Streaming server required.' ].
	(aRequestContext request includesKey: CTComet headerField)
		ifTrue: [ self cometForContext: aRequestContext ]
		ifFalse: [ super handleFiltered: aRequestContext ]! !

!CTFilter methodsFor: 'initialization' stamp: 'lr 11/3/2008 20:43'!
initialize
	super initialize.
	handlers := Dictionary new! !

!CTFilter methodsFor: 'testing' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ true! !

!CTFilter methodsFor: 'public' stamp: 'lr 3/5/2009 21:47'!
registerPusher: aPusher context: aContext
	| handler |
	handler := CTHandler on: aPusher.
	handlers at: handler id put: handler.
	^ handler id! !

!WARequestFilter methodsFor: '*comet-core' stamp: 'lr 11/3/2008 20:50'!
isCometFilter
	^ false! !

!WAStreamedResponse methodsFor: '*comet-core' stamp: 'lr 3/25/2009 13:45'!
flushComet
	^ externalStream flushComet! !

WAFileLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'scripts' stamp: 'lr 8/10/2009 21:19'!
cometJs
	^ '/* Seaside Comet API
 * (c) 2006-2009 Lukas Renggli (renggli@gmail.com)
 */

var Comet = function(url) {
	this.url = url;
	this.iframe = this.createIframe();
	document.body.appendChild(this.iframe);
};

Comet.prototype.connect = function() {
	this.iframe.onLoad = this.onLoad;
	this.iframe.src = this.url;
};

Comet.prototype.disconnect = function() {
	this.iframe.onLoad = this.iframe.src = null;
};

Comet.prototype.onLoad = function() {
	this.disconnect();
	this.connect();
};

Comet.prototype.createIframe = function() {
	var frame = document.createElement("iframe");
	frame.style.display = "block";
	frame.style.visibility = frame.style.overflow = "hidden";
	frame.style.width = frame.style.height = frame.style.border = frame.style.margin = frame.style.padding = "0";
	return frame;
};

Comet.eval = function(code) {
 	return eval(code);
};'! !

!CTLibrary methodsFor: 'accessing' stamp: 'lr 1/2/2008 20:11'!
selectorsToInclude
	^ #( cometJs )! !

WAObject subclass: #CTHandler
	instanceVariableNames: 'id pusher session response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTHandler class methodsFor: 'instance creation' stamp: 'lr 3/5/2009 21:47'!
on: aPusher
	^ self basicNew initializeOn: aPusher! !

!CTHandler methodsFor: 'private' stamp: 'jf 2/5/2010 22:56'!
basicConnect: aRequestContext
	session := aRequestContext session.
	response := aRequestContext response.
	response
		doNotCache;
		contentType: aRequestContext application contentType;
		nextPutAll: (self preambleFor: aRequestContext);
		nextPutAll: ((String new: 1024) atAllPut: Character space)! !

!CTHandler methodsFor: 'private' stamp: 'lr 3/5/2009 21:36'!
basicDisconnect
	response ifNil: [ ^ self ].
	response destroy.
	response := nil! !

!CTHandler methodsFor: 'private' stamp: 'lr 10/28/2007 14:45'!
basicTerminate
	thisContext swapSender: nil.
	Processor activeProcess terminate! !

!CTHandler methodsFor: 'actions' stamp: 'lr 3/5/2009 21:38'!
connect: aRequestContext
	self
		basicDisconnect;
		basicConnect: aRequestContext.
	self pusher register: self.
	self basicTerminate! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:22'!
id
	^ id! !

!CTHandler methodsFor: 'initialization' stamp: 'jf 3/13/2009 17:57'!
initializeOn: aPusher
	self initialize.
	id := WAKeyGenerator current 
		keyOfLength: 16.
	pusher := aPusher! !

!CTHandler methodsFor: 'private' stamp: 'lr 8/18/2010 22:18'!
notifySession
	self session application cache 
		notifyRetrieved: self session 
		key: self session key! !

!CTHandler methodsFor: 'private' stamp: 'jf 2/5/2010 22:56'!
preambleFor: aRequestContext
	^ '<!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en"><head><title>Comet</title></head><body>'
		, ((String new: 1024) atAllPut: Character space)! !

!CTHandler methodsFor: 'actions' stamp: 'lr 8/18/2010 22:19'!
push: aString
	"Push aString to the receiver. Notify the session about the update, so that it does not timeout. Answer true, if the connection is still valid."

	| success |
	response isNil
		ifTrue: [ ^ false ].
	success := [ response nextPutAll: aString; flushComet ]
		on: Error
		do: [ :err | ^ false ].
	success
		ifTrue: [ self notifySession ].
	^ success! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 7/12/2006 14:24'!
pusher
	^ pusher! !

!CTHandler methodsFor: 'accessing' stamp: 'lr 3/17/2009 20:26'!
session
	^ session! !

WAObject subclass: #CTPusher
	instanceVariableNames: 'mutex handlers rendererClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTPusher class
	instanceVariableNames: 'process'!

CTPusher class
	instanceVariableNames: 'process'!

!CTPusher class methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
initialize
	GRPlatform current
		addToStartUpList: self;
		addToShutDownList: self! !

!CTPusher class methodsFor: 'private' stamp: 'jf 2/9/2010 01:26'!
pingProcess
	self allInstancesDo: [ :each | each ping ].
	GRPlatform current cometWait! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
shutDown
	process ifNil: [ ^ self ].
	GRPlatform current terminateProcess: process.
	process := nil! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 2/9/2010 01:26'!
startUp
	self shutDown.
	process := GRPlatform current cometForkProcessDo: [ [ self pingProcess ] repeat ]! !

!CTPusher class methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
unload
	GRPlatform current
		removeFromStartUpList: self;
		removeFromShutDownList: self.
	self shutDown! !

!CTPusher methodsFor: 'accessing' stamp: 'lr 7/12/2006 11:50'!
handlers
	"Answer the current list of handlers."

	^ handlers! !

!CTPusher methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
initialize
	super initialize.
	mutex := GRPlatform current semaphoreClass forMutualExclusion.
	handlers := Array new! !

!CTPusher methodsFor: 'public' stamp: 'obi 6/23/2010 19:37'!
javascript: aBlock
	"Evaluate aBlock with an JSScript instance and pushes the resulting script-string aString to all associated handlers."

	| script |
	script := (JSScript context: self renderContext)
		rendererClass: self rendererClass;
		yourself.
	aBlock value: script.
	self push: (String streamContents: [ :stream |
		stream
			nextPutAll: '<script type="text/javascript">parent.Comet.eval(';
			javascript: script asJavascript;
			nextPutAll: ')</script>' ])! !

!CTPusher methodsFor: 'accessing' stamp: 'jf 2/5/2010 23:00'!
mutex
	"Answer the mutex in use with the receiver."

	^ mutex! !

!CTPusher methodsFor: 'protected' stamp: 'lr 3/25/2009 13:47'!
ping
	"Ping all the associated handlers."

	self push: ' '! !

!CTPusher methodsFor: 'protected' stamp: 'lr 3/25/2009 13:47'!
push: aString
	"Push aString to all associated handlers."

	self mutex critical: [ handlers := handlers select: [ :each | each push: aString ] ]! !

!CTPusher methodsFor: 'protected' stamp: 'lr 8/30/2006 10:43'!
register: aHandler
	"Register a new handler into this pusher."

	mutex critical: [ handlers := handlers copyWith: aHandler ]! !

!CTPusher methodsFor: 'accessing-rendering' stamp: 'lr 8/11/2009 10:46'!
renderContext
	"Answer a fake rendering context for this pusher."

	^ WARenderContext new
		document: (WAHtmlDocument on: (WriteStream on: String new));
		actionUrl: WAUrl new;
		yourself! !

!CTPusher methodsFor: 'accessing-rendering' stamp: 'lr 8/11/2009 10:46'!
rendererClass
	"Answer the default renderer for this pusher."

	^ rendererClass ifNil: [ WARenderer default ]! !

!CTPusher methodsFor: 'accessing-rendering' stamp: 'lr 8/11/2009 10:47'!
rendererClass: aRendererClass
	rendererClass := aRendererClass! !

!GRCodecStream methodsFor: '*comet-core' stamp: 'lr 3/25/2009 13:45'!
flushComet
	^ stream flushComet! !

WAError subclass: #CTStreamingServerRequired
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTStreamingServerRequired methodsFor: 'accessing' stamp: 'jf 1/11/2010 12:21'!
seasidePossibleCauses
	^ #(
		'The ServerAdaptor handling this request does not support the streaming of HTTP responses, which is required when using Comet. On Squeak/Pharo, you can use WAListenerAdaptor in the Seaside-Adaptors-Comanche package.'
		)! !

JSObject subclass: #CTComet
	instanceVariableNames: 'pusher url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTComet class methodsFor: 'configuration' stamp: 'lr 7/12/2006 12:10'!
headerField
	^ '_c'! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
connect
	self call: 'connect'! !

!CTComet methodsFor: 'private' stamp: 'jf 2/15/2010 02:08'!
defaultUrl
	^ self renderContext actionUrl
		addField: self class headerField
		value: self pusher id! !

!CTComet methodsFor: 'actions' stamp: 'lr 7/3/2006 13:54'!
disconnect
	self call: 'disconnect'! !

!CTComet methodsFor: 'accessing-readonly' stamp: 'jf 7/19/2009 14:19'!
filter
	^ self session filters
		detect: [ :each | each isCometFilter ]
		ifNone: [ self session addFilterFirst: CTFilter new ]! !

!CTComet methodsFor: 'printing' stamp: 'lr 12/24/2009 10:16'!
javascriptContentOn: aStream
	aStream
		nextPutAll: 'new Comet(';
		javascript: self url;
		nextPut: $)! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:44'!
pusher
	^ pusher! !

!CTComet methodsFor: 'accessing' stamp: 'jf 2/15/2010 02:12'!
pusher: aPusher
	self url: (self renderContext actionUrl
		withField: self class headerField
		value: (self filter
			registerPusher: (pusher := aPusher)
			context: self requestContext))! !

!CTComet methodsFor: 'accessing' stamp: 'lr 7/3/2006 13:33'!
url
	^ url ifNil: [ url := self defaultUrl ]! !

!CTComet methodsFor: 'accessing' stamp: 'lr 9/8/2009 12:50'!
url: aUrl
	url := aUrl! !

!WARenderer methodsFor: '*comet-core' stamp: 'jf 7/19/2009 18:43'!
comet
	^ (CTComet context: self context)
		rendererClass: self species;
		yourself! !

!GRPackage class methodsFor: '*comet-core' stamp: 'lr 2/17/2010 15:10'!
cometCore
	^ self new
		name: 'Comet-Core';
		description: 'Server push, stream events from the server to client.';
		addDependency: 'Javascript-Core';
		addDependency: 'Seaside-Canvas';
		url: #seasideUrl;
		yourself! !
CTPusher initialize!
